<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\Support\GenericPropertyObject;
use SimpleXMLIterator;

class Item extends GenericPropertyObject
{

    /**
     * @var SimpleXMLIterator
     */
    protected $el;

    public function __construct(SimpleXMLIterator $el)
    {
        $this->fill(Arr::get((array)$el->attributes(), '@attributes', []));

        $this->el = $el;
    }

    /**
     * Filter the content of the blocks from unwanted elements if allowedTags is set.
     * This function will be applied to every block
     *
     * @param string $text
     * @param array  $allowedTags
     * @param array  $forbiddenTags
     *
     * @return string
     * @since 0.1
     */
    public function filterInlineContent(string $text = '', array $allowedTags = [], array $forbiddenTags = []): string
    {

        if ( ! empty($forbiddenTags)) {
            foreach ($forbiddenTags as $tag) {
                $text = preg_replace('/\<'. $tag .'(.*?)\>(.*?)\<\/'. $tag .'\>/', '', $text);
            }
        }

        if ( ! empty($allowedTags)) {
            return trim(strip_tags($text, '<' . implode('><', $allowedTags) . '>'));
        }

        return $text;
    }

    public function getItemAttributes(): array
    {
        return $this->getProperties();
    }

    public function getItemAttribute(string $key = '', $default = '')
    {
        $itemAttributes = $this->getItemAttributes();
        if (array_key_exists($key, $itemAttributes)) {
            return $itemAttributes[$key];
        }

        return $default;
    }

    public function getType(): string
    {
        return Arr::get($this->getItemAttributes(), 'type', '');
    }

    public function getFirstChild(string $name): ?Item
    {
        $items = $this->getChildren($name);

        return array_shift($items);
    }

    public function getChild(string $name): array
    {
        return $this->getChildren($name);
    }

    public function getFirstChildAsXml(string $key, array $allowedTags = [], array $forbiddenTags = []): string
    {
        $element = $this->getFirstChild($key);

        return $element instanceof self ? $element->getFilteredContent($allowedTags, $forbiddenTags) : '';
    }

    public function removeLinebreak(string $xml): string
    {
        return implode('', array_map('trim', explode(PHP_EOL, $xml)));
    }

    public function getContentAsXML(): string
    {
        return $this->removeLinebreak($this->el->asXML());
    }

    public function getFilteredContent(array $allowedTags = [], array $forbiddenTags = []): string
    {
        return $this->filterInlineContent($this->getContentAsXML(), $allowedTags, $forbiddenTags);
    }

    /**
     * Returns all none empty children of the specified node type.
     *
     * @param string $nodeName
     *
     * @return array of Item
     */
    private function getChildren(string $nodeName): array
    {
        $items = [];
        if (isset($this->el->$nodeName)) {
            foreach ($this->el->$nodeName as $node) {
                //Clone node so we can unset empty children
                $nodeClone = clone $node;
                //Check node for empty children
                foreach($node as $key => $child) {
                    if(!$child) {
                        unset($nodeClone->$key);
                    }
                }
                if($nodeClone instanceof SimpleXMLIterator) {
                    $item = new Item($nodeClone);
                    $item->fill((array)$item->el);

                    $items[] = $item;
                }
            }
        }

        return $items;
    }
}
