<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use function Spec\newsML\loadNewsMLFile;

/**
 * @method parse(\SimpleXMLElement $simplexml_load_string)
 * @method addForbiddenTags(string[] $array)
 * @method setForbiddenTags(string[] $array)
 * @method setAllowedTags(string[] $array)
 */
class ElementParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ElementParser::class);
    }

    public function it_should_parse_an_element()
    {
        $xml = <<<XML
<element>
    <p>Tagless</p>
    <i>Tagged</i>
    <em>Tagged</em>
    <div>Tagless</div>
</element>
XML;
        $this->shouldBeParsedTo(
            $xml,
            'Tagless<i>Tagged</i><em>Tagged</em>Tagless'
        );
    }

    public function it_should_allow_tags()
    {
        $xml = <<<XML
<element>
    <span>Tagged</span>
    <i>Tagged</i>
    <em>Tagged</em>
    <b>Tagged</b>
    <strong>Tagged</strong>
    <sup>Tagged</sup>
    <sub>Tagged</sub>
    <br>Tagged</br>
    <hr>Tagged</hr>
    <a>Tagged</a>
    <script>Tagged</script>
    <iframe>Tagged</iframe>
</element>
XML;
        $this->shouldBeParsedTo(
            $xml,
            '<span>Tagged</span><i>Tagged</i><em>Tagged</em><b>Tagged</b><strong>Tagged</strong><sup>Tagged</sup><sub>Tagged</sub><br>Tagged</br><hr>Tagged</hr><a>Tagged</a><script>Tagged</script><iframe>Tagged</iframe>'
        );
    }

    public function it_should_remove_forbidden_tags()
    {
        $this->setForbiddenTags(['ins']);

        $xml = <<<XML
<element>Here is a paragraph <span>with allowed tag.</span>
<ins id="dsg">This tag should be removed!!!</ins> Another paragraph.<ins id="dssdsdg">This should also be removed.</ins> End.
</element>
XML;
        $this->shouldBeParsedTo(
            $xml,
            'Here is a paragraph <span>with allowed tag.</span> Another paragraph. End.'
        );
    }

    public function it_should_override_allowed_tags()
    {
        $this->setAllowedTags(['span', 'i', 'em', 'strong']);

        $xml = <<<XML
<element>
    <span>Tagged</span>
    <i>Tagged</i>
    <em>Tagged</em>
    <b>Tagged</b>
    <strong>Tagged</strong>
    <sup>Tagged</sup>
    <sub>Tagged</sub>
    <br>Tagged</br>
    <hr>Tagged</hr>
    <a>Tagged</a>
    <script>Tagged</script>
    <iframe>Tagged</iframe>
</element>
XML;
        $this->shouldBeParsedTo(
            $xml,
            '<span>Tagged</span><i>Tagged</i><em>Tagged</em>Tagged<strong>Tagged</strong>TaggedTaggedTaggedTaggedTaggedTaggedTagged'
        );
    }

    private function shouldBeParsedTo($xml, $parsedContent)
    {
        $this->parse(simplexml_load_string($xml, 'SimpleXMLIterator'))->shouldHaveContent($parsedContent);
    }

    public function getMatchers(): array
    {
        return [
            'haveContent' => function (array $subjects, string $content) {
                if (empty($subjects)) {
                    throw new FailureException('Did not return any parsed items . ');
                }

                $subject = $subjects[0];

                if ( ! $subject instanceof Item) {
                    throw new FailureException(sprintf('Failed to parse the object into a "%s" . ', Item::class));
                }
                if ($subject->content !== $content) {
                    throw new FailureException(implode(PHP_EOL, [
                            'The Item did not contain desired content . ',
                            'expected ',
                            $content,
                            'but got ',
                            $subject->content
                        ])
                    );
                }

                return true;
            },
        ];
    }
}
