<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\TeaserParser;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use SimpleXMLIterator;

class TeaserParserSpec extends ObjectBehavior
{

    function it_is_initializable()
    {
        $this->shouldHaveType(TeaserParser::class);
    }

    public function it_should_parse_an_element()
    {
        $xml = '
            <object id="MjQ0LDI4LDIxNSwxNzk" title="headlineObject" type="x-im/teaser">
                <data>
                    <title>headline</title>
                    <text>thetext</text>
                </data>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'id' => 'MjQ0LDI4LDIxNSwxNzk',
                'title' => 'headlineObject',
                'type' => 'x-im/teaser',
                'headline' => 'headline',
                'text' => 'thetext'
            ]);
    }

    public function it_should_parse_an_element_with_links()
    {
        $xml = '
            <object id="MjQ0LDI4LDIxNSwxNzk" title="headlineObject" type="x-im/teaser">
                <data>
                    <title>headline</title>
                    <text>thetext</text>
                </data>
                <links>
                    <link rel="image" type="x-im/image" uri="im://image/YaJGeEHyHO_2LnZ154NA-5XS6e8.jpg" uuid="b4909f8c-1b07-5352-99d9-40f7c8d8a7ef">
                        <data>
                            <width>1378</width>
                            <height>578</height>
                        </data>
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeysLinks([
                'rel' => 'image',
                'type' => 'x-im/image',
                'uri' => 'im://image/YaJGeEHyHO_2LnZ154NA-5XS6e8.jpg',
                'uuid' => 'b4909f8c-1b07-5352-99d9-40f7c8d8a7ef',
                'width' => '1378',
                'height' => '578'
            ]);
    }

    public function it_should_not_set_default_data_if_data_is_empty()
    {
        $xml = '
            <object id="MjQ0LDI4LDIxNSwxNzk" title="headlineObject" type="x-im/teaser">
                <data></data>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'id' => 'MjQ0LDI4LDIxNSwxNzk',
                'title' => 'headlineObject',
                'type' => 'x-im/teaser',
            ]);
    }

    public function it_should_have_text_styles()
    {
        $xml = '
        <object id="Nyw3MSwzOCwyMTY" type="x-im/teaser" title="Headline text without markup">
            <data>
                <title>Headline text without markup</title>
                <text>
                    <strong>Text</strong> <em>text</em> with <mark>markup</mark>
                </text>
                <subject>
                    <em>Subject</em> <strong>text</strong> with <mark>markup</mark>
                </subject>
            </data>
        </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'id' => 'Nyw3MSwzOCwyMTY',
                'type' => 'x-im/teaser',
                'title' => 'Headline text without markup',
                'headline' => 'Headline text without markup',
                'text' => '<strong>Text</strong> <em>text</em> with <mark>markup</mark>',
                'subject' => '<em>Subject</em> <strong>text</strong> with <mark>markup</mark>'
            ]);
    }

    public function it_should_only_have_allowed_tags()
    {
        $xml = '
        <object id="Nyw3MSwzOCwyMTY" type="x-im/teaser" title="Headline text without markup">
            <data>
                <title>Headline text without markup</title>
                <text>
                    <td><strong>Text</strong></td> <em>text</em> with <mark>markup</mark><a href="www.google.com">link text</a>
                </text>
                <subject>
                    <div><em>Subject</em> <strong>text</strong> with <mark>markup</mark></div>
                </subject>
            </data>
        </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayValuesAndKeys([
                'id' => 'Nyw3MSwzOCwyMTY',
                'type' => 'x-im/teaser',
                'title' => 'Headline text without markup',
                'headline' => 'Headline text without markup',
                'text' => '<strong>Text</strong> <em>text</em> with <mark>markup</mark>link text',
                'subject' => '<em>Subject</em> <strong>text</strong> with <mark>markup</mark>'
            ]);
    }

    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/teaser');
    }

    public function getMatchers(): array
    {
        return [
            'haveArrayValuesAndKeys' => static function ($subjects, $values) {
                if (empty($subjects)) {
                    throw new FailureException('Did not return any parsed items.');
                }

                $subject = $subjects[0];

                if ( ! $subject instanceof Item) {
                    throw new FailureException(sprintf('Failed to parse the object into a "%s".', Item::class));
                }

                foreach ($values as $key => $value) {
                    if ( ! $subject->offsetExists($key)) {
                        throw new FailureException(implode(PHP_EOL, [
                                "The Item did not contain desired property: \"{$key}\". ",
                                'expected ',
                                json_encode($values, JSON_PRETTY_PRINT),
                                'but got ',
                                $subject->toJson(JSON_PRETTY_PRINT)
                            ])
                        );
                    }

                    if ($subject->get($key) !== $value) {
                        throw new FailureException(implode(PHP_EOL, [
                                "The Item property \"{$key}\" did not contain desired value: \"{$value}\". ",
                                'expected ',
                                json_encode($values, JSON_PRETTY_PRINT),
                                'but got ',
                                $subject->toJson(JSON_PRETTY_PRINT)
                            ])
                        );
                    }
                }

                return true;
            },
            'haveArrayValuesAndKeysLinks' => static function ($subjects, $values) {
                if (empty($subjects)) {
                    throw new FailureException('Did not return any parsed items.');
                }

                $item = $subjects[0];

                if ( ! $item instanceof Item) {
                    throw new FailureException(sprintf('Failed to parse the object into a "%s".', Item::class));
                }

                $links = $item->getMultiValue('links');

                if (empty($links)) {
                    throw new FailureException('Item did not contain any links.');
                }

                $image = $links[0];

                if ( ! $image instanceof ImageItem) {
                    throw new FailureException(sprintf('Item did not contain "%s".', ImageItem::class));
                }

                foreach ($values as $key => $value) {
                    if ( ! $image->offsetExists($key)) {
                        throw new FailureException(implode(PHP_EOL, [
                                "The Item did not contain desired property: \"{$key}\". ",
                                'expected ',
                                json_encode($values, JSON_PRETTY_PRINT),
                                'but got ',
                                $image->toJson(JSON_PRETTY_PRINT)
                            ])
                        );
                    }

                    if ($image->get($key) !== $value) {
                        throw new FailureException(implode(PHP_EOL, [
                                "The Item property \"{$key}\" did not contain desired value: \"{$value}\". ",
                                'expected ',
                                json_encode($values, JSON_PRETTY_PRINT),
                                'but got ',
                                $image->toJson(JSON_PRETTY_PRINT)
                            ])
                        );
                    }
                }

                return true;
            },
        ];
    }
}
